@@ -1,9 +1,40 @@
+/* 
+ *
+ * This program is free software; you can redistribute it and/or
+ * modify it under the terms of the GNU General Public License as
+ * published by the Free Software Foundation; either version 2 of the
+ * License, or (at your option) any later version.
+ *
+ * This program is distributed in the hope that it will be useful, but
+ * is provided AS IS, WITHOUT ANY WARRANTY; without even the implied
+ * warranty of MERCHANTABILITY, FITNESS FOR A PARTICULAR PURPOSE, and
+ * NON-INFRINGEMENT.  See the GNU General Public License for more details.
+ *
+ * You should have received a copy of the GNU General Public License
+ * along with this program; if not, write to the Free Software
+ * Foundation, Inc., 59 Temple Place - Suite 330, Boston,
+ * MA 02111-1307, USA.
+ *
+ * In addition, as a special exception, the copyright holders give
+ * permission to link the code of portions of this program with the
+ * OpenSSL library under certain conditions as described in each
+ * individual source file, and distribute linked combinations
+ * including the two.
+ * You must obey the GNU General Public License in all respects
+ * for all of the code used other than OpenSSL.  If you modify
+ * file(s) with this exception, you may extend this exception to your
+ * version of the file(s), but you are not obligated to do so.  If you
+ * do not wish to do so, delete this exception statement from your
+ * version.  If you delete this exception statement from all source
+ * files in the program, then also delete it here.
+ */
+
 #ifndef _AIRODUMP_NG_H_
 #define _AIRODUMP_NG_H_
 
-/* some constants */
+#include "eapol.h"
 
-#define MAX_IE_ELEMENT_SIZE 256
+/* some constants */
 
 #define REFRESH_RATE 100000  /* default delay in us between updates */
 #define DEFAULT_HOPFREQ 250  /* default delay in ms between channel hopping */
@@ -86,7 +117,6 @@
 //milliseconds to store last packets
 #define BUFFER_TIME 3000
 
-extern char * getVersion(char * progname, int maj, int min, int submin, int svnrev, int beta, int rc);
 extern unsigned char * getmac(char * macAddress, int strict, unsigned char * mac);
 extern int get_ram_size(void);
 char *get_manufacturer(unsigned char mac0, unsigned char mac1, unsigned char mac2);
@@ -105,13 +135,27 @@ char *f_ext[NB_EXTENSIONS] = { AIRODUMP_NG_CSV_EXT, AIRODUMP_NG_GPS_EXT, AIRODUM
 extern const unsigned long int crc_tbl[256];
 extern const unsigned char crc_chop_tbl[256][4];
 
-static uchar ZERO[32] =
+static unsigned char ZERO[32] =
 "\x00\x00\x00\x00\x00\x00\x00\x00"
 "\x00\x00\x00\x00\x00\x00\x00\x00"
 "\x00\x00\x00\x00\x00\x00\x00\x00"
 "\x00\x00\x00\x00\x00\x00\x00\x00";
 
-#define OUI_PATH "/usr/local/etc/aircrack-ng/airodump-ng-oui.txt"
+const char *OUI_PATHS[] = {
+    "./airodump-ng-oui.txt",
+    "/etc/aircrack-ng/airodump-ng-oui.txt",
+    "/usr/local/etc/aircrack-ng/airodump-ng-oui.txt",
+    "/usr/share/aircrack-ng/airodump-ng-oui.txt",
+    "/var/lib/misc/oui.txt",
+    "/usr/share/misc/oui.txt",
+    "/var/lib/ieee-data/oui.txt",
+    "/usr/share/ieee-data/oui.txt",
+    "/etc/manuf/oui.txt",
+    "/usr/share/wireshark/wireshark/manuf/oui.txt",
+    "/usr/share/wireshark/manuf/oui.txt",
+    NULL
+};
+
 #define MIN_RAM_SIZE_LOAD_OUI_RAM 32768
 
 int read_pkts=0;
@@ -119,10 +163,11 @@ int read_pkts=0;
 int abg_chans [] =
 {
     1, 7, 13, 2, 8, 3, 14, 9, 4, 10, 5, 11, 6, 12,
-    36, 40, 44, 48, 52, 56, 60, 64, 100, 104, 108,
-    112, 116, 120, 124, 128, 132, 136, 140, 149,
-    153, 157, 161, 184, 188, 192, 196, 200, 204,
-    208, 212, 216,0
+    36, 38, 40, 42, 44, 46, 48, 50, 52, 54, 56, 58,
+    60, 62, 64, 100, 102, 104, 106, 108, 110, 112,
+    114, 116, 118, 120, 122, 124, 126, 128, 132,
+    134, 136, 138, 140, 142, 144, 149, 151, 153,
+    155, 157, 159, 161, 165, 169, 173, 0
 };
 
 int bg_chans  [] =
@@ -132,10 +177,11 @@ int bg_chans  [] =
 
 int a_chans   [] =
 {
-    36, 40, 44, 48, 52, 56, 60, 64, 100, 104, 108,
-    112, 116, 120, 124, 128, 132, 136, 140, 149,
-    153, 157, 161, 184, 188, 192, 196, 200, 204,
-    208, 212, 216,0
+    36, 38, 40, 42, 44, 46, 48, 50, 52, 54, 56, 58,
+    60, 62, 64, 100, 102, 104, 106, 108, 110, 112,
+    114, 116, 118, 120, 122, 124, 126, 128, 132,
+    134, 136, 138, 140, 142, 144, 149, 151, 153,
+    155, 157, 159, 161, 165, 169, 173, 0
 };
 
 int *frequencies;
@@ -156,6 +202,56 @@ struct oui {
 	struct oui *next;
 };
 
+/* WPS_info struct */
+struct WPS_info {
+    unsigned char version;    /* WPS Version */
+    unsigned char state;      /* Current WPS state */
+    unsigned char ap_setup_locked; /* AP setup locked */
+    unsigned int meth;        /* WPS Config Methods */
+};
+
+#define MAX_AC_MCS_INDEX 8
+
+/* 802.11n channel information */
+struct n_channel_info {
+	char mcs_index;     /* Maximum MCS TX index     */
+    char sec_channel;            /* 802.11n secondary channel*/
+    unsigned char short_gi_20;   /* Short GI for 20MHz       */
+    unsigned char short_gi_40;   /* Short GI for 40MHz       */
+    unsigned char any_chan_width;/* Support for 20 or 40MHz  
+                                    as opposed to only 20 or 
+                                    only 40MHz               */
+};
+
+/* 802.11ac channel information */
+struct ac_channel_info {
+    unsigned char center_sgmt[2];
+                                 /* 802.11ac Center segment 0*/
+    unsigned char mu_mimo;       /* MU-MIMO support          */
+    unsigned char short_gi_80;   /* Short GI for 80MHz       */
+	unsigned char short_gi_160;  /* Short GI for 160MHz      */
+	unsigned char split_chan;    /* 80+80MHz Channel support */
+	unsigned char mhz_160_chan;  /* 160 MHz channel support  */
+    unsigned char wave_2;        /* Wave 2                   */
+	unsigned char mcs_index[MAX_AC_MCS_INDEX];
+	                             /* Maximum TX rate          */
+};
+
+enum channel_width_enum {
+	CHANNEL_UNKNOWN_WIDTH,
+	CHANNEL_3MHZ,
+	CHANNEL_5MHZ,
+	CHANNEL_10MHZ,
+	CHANNEL_20MHZ,
+	CHANNEL_22MHZ,
+	CHANNEL_30MHZ,
+	CHANNEL_20_OR_40MHZ,
+	CHANNEL_40MHZ,
+	CHANNEL_80MHZ,
+	CHANNEL_80_80MHZ,
+	CHANNEL_160MHZ
+};
+
 /* linked list of detected access points */
 struct AP_info
 {
@@ -165,6 +261,13 @@ struct AP_info
     time_t tinit, tlast;      /* first and last time seen */
 
     int channel;              /* AP radio channel         */
+    enum channel_width_enum
+		channel_width;        /* Channel width            */
+    char standard[3];         /* 802.11 standard: n or ac */
+	struct n_channel_info
+		n_channel;            /* 802.11n channel info     */
+	struct ac_channel_info
+		ac_channel;           /* 802.11ac channel info    */
     int max_speed;            /* AP maximum speed in Mb/s */
     int avg_power;            /* averaged signal power    */
     int best_power;           /* best signal power    */
@@ -191,6 +294,8 @@ struct AP_info
     char *manuf;              /* the access point's manufacturer */
     unsigned char essid[MAX_IE_ELEMENT_SIZE];
                               /* ascii network identifier */
+    unsigned long long timestamp;
+    						  /* Timestamp to calculate uptime   */
 
     unsigned char lanip[4];   /* last detected ip address */
                               /* if non-encrypted network */
@@ -223,18 +328,7 @@ struct AP_info
 					  
     int marked;
     int marked_color;
-};
-
-struct WPA_hdsk
-{
-    uchar stmac[6];				 /* supplicant MAC               */
-    uchar snonce[32];			 /* supplicant nonce             */
-    uchar anonce[32];			 /* authenticator nonce          */
-    uchar keymic[16];			 /* eapol frame MIC              */
-    uchar eapol[256];			 /* eapol frame contents         */
-    int eapol_size;				 /* eapol frame size             */
-    int keyver;					 /* key version (TKIP / AES)     */
-    int state;					 /* handshake completion         */
+    struct WPS_info wps;
 };
 
 /* linked list of detected clients */
@@ -261,6 +355,8 @@ struct ST_info
     struct WPA_hdsk wpa;     /* WPA handshake data        */
     int qos_to_ds;           /* does it use 802.11e to ds */
     int qos_fr_ds;           /* does it receive 802.11e   */
+    int channel;             /* Channel station is seen   */
+                             /*  Not used yet		  */
 };
 
 /* linked list of detected macs through ack, cts or rts frames */
@@ -294,6 +390,11 @@ struct globals
     unsigned char prev_bssid[6];
     unsigned char f_bssid[6];
     unsigned char f_netmask[6];
+    char **f_essid;
+    int f_essid_count;
+#ifdef HAVE_PCRE
+    pcre *f_essid_regex;
+#endif
     char *dump_prefix;
     char *keyout;
     char *f_cap_name;
@@ -417,7 +518,18 @@ struct globals
     pthread_mutex_t mx_print;			 /* lock write access to ap LL   */
     pthread_mutex_t mx_sort;			 /* lock write access to ap LL   */
     
-    uchar selected_bssid[6];	/* bssid that is selected */
+    unsigned char selected_bssid[6];	/* bssid that is selected */
+
+    int ignore_negative_one;
+    u_int maxsize_essid_seen;
+    int show_manufacturer;
+    int show_uptime;
+    int file_write_interval;
+    u_int maxsize_wps_seen;
+    int show_wps;
+#ifdef CONFIG_LIBNL
+    int htval;
+#endif
 }
 G;
 
