@@ -1,7 +1,7 @@
 /*
  *  Common functions for all aircrack-ng tools
  *
- *  Copyright (C) 2006, 2007, 2008, 2009 Thomas d'Otreppe
+ *  Copyright (C) 2006-2018 Thomas d'Otreppe <tdotreppe@aircrack-ng.org>
  *
  *  This program is free software; you can redistribute it and/or modify
  *  it under the terms of the GNU General Public License as published by
@@ -16,6 +16,20 @@
  *  You should have received a copy of the GNU General Public License
  *  along with this program; if not, write to the Free Software
  *  Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
+ *
+ *
+ *  In addition, as a special exception, the copyright holders give
+ *  permission to link the code of portions of this program with the
+ *  OpenSSL library under certain conditions as described in each
+ *  individual source file, and distribute linked combinations
+ *  including the two.
+ *  You must obey the GNU General Public License in all respects
+ *  for all of the code used other than OpenSSL. *  If you modify
+ *  file(s) with this exception, you may extend this exception to your
+ *  version of the file(s), but you are not obligated to do so. *  If you
+ *  do not wish to do so, delete this exception statement from your
+ *  version. *  If you delete this exception statement from all source
+ *  files in the program, then also delete it here.
  */
 
 #include <stdio.h>
@@ -27,18 +41,143 @@
 #include <dirent.h>
 #include <unistd.h>
 #include <ctype.h>
-
+#if defined(__FreeBSD__) || defined(__OpenBSD__) || defined(__DragonFly__)
+#include <sys/sysctl.h>
+#include <sys/user.h>
+#endif
+#if (defined(_WIN32) || defined(_WIN64)) || defined(__CYGWIN32__)
+#include <io.h>
+#include <windows.h>
+#include <errno.h>
+#endif
 #define isHex(c) (hexToInt(c) != -1)
 #define HEX_BASE 16
 
+/*
+ * The following function comes from jumbo.c from JTR.
+ * It has the following license:
+ *
+ * This file is Copyright (c) 2013-2014 magnum, Lukasz and JimF,
+ * and is hereby released to the general public under the
+following terms:
+ * Redistribution and use in source and binary forms, with or
+without
+ * modifications, are permitted.
+*/
+#if defined (__CYGWIN32__) && !defined(__CYGWIN64__)
+int fseeko64(FILE* fp, int64_t offset, int whence) {
+	fpos_t pos;
+
+	if (whence == SEEK_CUR) {
+		if (fgetpos (fp, &pos))
+			return (-1);
+
+		pos += (fpos_t) offset;
+	} else if (whence == SEEK_END) {
+		/* If writing, we need to flush before getting file length. */
+		long long size;
+
+		fflush (fp);
+		size = 0;
+
+		GetFileSizeEx((HANDLE)_get_osfhandle(fileno(fp)), (PLARGE_INTEGER)&size);
+		pos = (fpos_t) (size + offset);
+	} else if (whence == SEEK_SET)
+		pos = (fpos_t) offset;
+	else {
+		errno = EINVAL;
+		return (-1);
+	}
+
+	return fsetpos (fp, &pos);
+}
+
+int64_t ftello64(FILE * fp) {
+	fpos_t pos;
+
+	if (fgetpos(fp, &pos))
+		return  -1LL;
+
+	return (int64_t)pos;
+}
+#endif
+
+/*
+ * Print the time and percentage in readable format
+ */
+void calctime(time_t t, float perc) {
+	int days = 0, hours = 0, mins = 0, secs = 0, remain = 0, printed = 0;
+	char buf[8];
+
+	days    = t / 86400;
+	remain  = t % 86400;
+
+	hours   = remain / 3600;
+	remain  = remain % 3600;
+
+	mins    = remain / 60;
+	secs    = remain % 60;
+
+	if (days)
+		printed += printf("%d day%s, ", days, (days>1?"s":""));
+
+	if (hours)
+		printed += printf("%d hour%s, ", hours, (hours>1?"s":""));
+
+	if (mins)
+		printed += printf("%d minute%s, ", mins, (mins>1?"s":""));
+
+	snprintf(buf, sizeof(buf), "%3.2f%%", perc);
+
+	printed += printf("%d second%s", secs, (secs!=1?"s":""));
+
+	printf("%*s %s\n", (int)(47-(printed+strlen(buf)%5))," ", buf);
+}
+
+int is_string_number(const char * str)
+{
+	int i;
+	if (str == NULL) {
+		return 0;
+	}
+
+	if (*str != '-' && !(isdigit((int)(*str)))) {
+		return 0;
+	}
+
+	for (i = 1; str[i] != 0; i++) {
+		if (!isdigit((int)(str[i]))) {
+			return 0;
+		}
+	}
+
+	return 1;
+}
+
 int get_ram_size(void) {
+	int ret = -1;
+#if defined(__FreeBSD__)
+	int mib[] = { CTL_HW, HW_PHYSMEM };
+	size_t len;
+	unsigned long physmem;
+
+	len	= sizeof(physmem);
+
+	if (!sysctl(mib, 2, &physmem, &len, NULL, 0))
+		ret = (physmem/1024);	// Linux returns memory size in kB, so we want to as well.
+#elif defined(_WIN32) || defined(_WIN64)
+	MEMORYSTATUSEX statex;
+	statex.dwLength = sizeof(statex);
+
+	GlobalMemoryStatusEx(&statex);
+	ret = (int)(statex.ullTotalPhys/1024);
+#else
 	FILE *fp;
 	char str[256];
 	int val = 0;
-	int ret = -1;
 
 	if (!(fp = fopen("/proc/meminfo", "r"))) {
-		perror("fopen fails");
+		perror("fopen fails on /proc/meminfo");
 		return ret;
 	}
 
@@ -50,14 +189,15 @@ int get_ram_size(void) {
 	}
 
 	fclose(fp);
+#endif
 	return ret;
 }
 
 /* Return the version number */
-char * getVersion(char * progname, int maj, int min, int submin, int svnrev, int beta, int rc)
+char * getVersion(char * progname, int maj, int min, int submin, const char* rev, int beta, int rc)
 {
 	int len;
-	char * temp;
+	char * temp, *rtemp;
 	char * provis = calloc(1,20);
 	len = strlen(progname) + 200;
 	temp = (char *) calloc(1,len);
@@ -80,57 +220,99 @@ char * getVersion(char * progname, int maj, int min, int submin, int svnrev, int
 		memset(provis, 0, 20);
 	}
 
-	if (svnrev > 0) {
-		snprintf(provis, 20," r%d",svnrev);
+	if (rev) {
+		char *tmp = strdup(rev);
+
+		char *sep = strstr(tmp, "_");
+		if (sep)
+		{
+			++sep;
+		}
+		else
+		{
+			sep = "";
+		}
+
+    	char *search = strstr(sep, "rev-");
+		if (search)
+		{
+			search[3] = ' ';
+		}
+
+		snprintf(provis, 20," %s", search ? search : sep);
 		strncat(temp, provis, len - strlen(temp));
 		memset(provis, 0, 20);
+		free(tmp);
 	}
 
 	free(provis);
-	temp = realloc(temp, strlen(temp)+1);
-	return temp;
+	rtemp = realloc(temp, strlen(temp)+1);
+	return (rtemp) ? rtemp : temp;
 }
 
 // Return the number of cpu. If detection fails, it will return -1;
 int get_nb_cpus()
 {
-		// Optmization for windows: use GetSystemInfo()
-        char * s, * pos;
-        FILE * f;
         int number = -1;
 
-		// Reading /proc/cpuinfo is more reliable on current CPUs,
-		// so put it first and try the old method if this one fails
+#if defined(_WIN32) || defined(_WIN64)
+	SYSTEM_INFO sysinfo = {0};
+
+	GetSystemInfo(&sysinfo);
+
+	number = sysinfo.dwNumberOfProcessors;
+#elif defined(__linux__)
+        char * s, * pos;
+        FILE * f;
+	// Reading /proc/cpuinfo is more reliable on current CPUs,
+	// so put it first and try the old method if this one fails
         f = fopen("/proc/cpuinfo", "r");
+
         if (f != NULL) {
-				s = (char *)calloc(1, 81);
-				if (s != NULL) {
-					// Get the latest value of "processor" element
-					// and increment it by 1 and it that value
-					// will be the number of CPU.
-					number = -2;
-					while (fgets(s, 80, f) != NULL) {
-							pos = strstr(s, "processor");
-							if (pos == s) {
-									pos = strchr(s, ':');
-									number = atoi(pos + 1);
-							}
-					}
-					++number;
-					free(s);
+		s = (char *)calloc(1, 81);
+
+		if (s != NULL) {
+			// Get the latest value of "processor" element
+			// and increment it by 1 and it that value
+			// will be the number of CPU.
+			number = -2;
+
+			while (fgets(s, 80, f) != NULL) {
+				pos = strstr(s, "processor");
+
+				if (pos == s) {
+					pos = strchr(s, ':');
+					number = atoi(pos + 1);
 				}
-				fclose(f);
+			}
+
+			++number;
+			free(s);
+		}
+
+		fclose(f);
         }
+#elif defined(__FreeBSD__) || defined(__OpenBSD__) || defined(__DragonFly__)
+// Not sure about defined(__DragonFly__) || defined(__NetBSD__) || defined(__OpenBSD__) || defined(__APPLE__)
+	int mib[] = { CTL_HW, HW_NCPU };
+	size_t len;
+	unsigned long nbcpu;
+
+	len     = sizeof(nbcpu);
+
+	if (!sysctl(mib, 2, &nbcpu, &len, NULL, 0)) {
+		number = (int)nbcpu;
+	}
+#endif
 
         #ifdef _SC_NPROCESSORS_ONLN
         // Try the usual method if _SC_NPROCESSORS_ONLN exist
         if (number == -1) {
-
-			number   = sysconf(_SC_NPROCESSORS_ONLN);
-			/* Fails on some archs */
-			if (number < 1) {
-				number = -1;
-			}
+		number   = sysconf(_SC_NPROCESSORS_ONLN);
+		/* Fails on some archs */
+		if (number < 1) {
+			number = -1;
+		}
         }
         #endif
 
@@ -250,31 +432,46 @@ int hexCharToInt(unsigned char c)
 	return table[c];
 }
 
-int hexStringToHex(char* in, int length, unsigned char* out)
+// in: input string
+// in_length: length of the string
+// out: output string (needs to be already allocated).
+// out_length: length of the array
+// returns amount of bytes saved to 'out' or -1 if an error happened
+int hexStringToArray(char* in, int in_length, unsigned char* out, int out_length)
 {
-    int i=0;
-    int char1, char2;
+    int i, out_pos;
+    int chars[2];
 
     char *input=in;
     unsigned char *output=out;
 
-    if(length < 1)
-        return 1;
+    if (in_length < 2 || out_length < (in_length / 3) + 1 || input == NULL || output == NULL)
+    	return -1;
 
-    for(i=0; i<length; i+=2)
+    out_pos = 0;
+    for (i = 0; i < in_length - 1; ++i)
     {
-        if(input[i] == '-' || input[i] == ':' || input[i] == '_' || input[i] == ' ')
-        {
-            input++;
-            length--;
-        }
-        char1 = hexCharToInt(input[i]);
-        char2 = hexCharToInt(input[i+1]);
-        if(char1 < 0 || char1 > 15)
+		if(input[i] == '-' || input[i] == ':' || input[i] == '_' || input[i] == ' ' || input[i] == '.')
+		{
+			continue;
+		}
+		// Check output array is big enough
+    	if(out_pos >= out_length)
+		{
+			return -1;
+		}
+    	chars[0] = hexCharToInt(input[i]);
+        // If first char is invalid (or '\0'), don't bother continuing (and you really shouldn't).
+        if(chars[0] < 0 || chars[0] > 15)
+        	return -1;
+
+        chars[1] = hexCharToInt(input[++i]);
+        // It should always be a multiple of 2 hex characters with or without separator
+        if(chars[1] < 0 || chars[1] > 15)
             return -1;
-        output[i/2] = ((char1 << 4) + char2) & 0xFF;
+        output[out_pos++] = ((chars[0] << 4) + chars[1]) & 0xFF;
     }
-    return (i/2);
+    return out_pos;
 }
 
 //Return the mac address bytes (or null if it's not a mac address)
@@ -296,6 +493,9 @@ int getmac(char * macAddress, int strict, unsigned char * mac)
 
 	while (macAddress[i] != 0)
 	{
+		if (macAddress[i] == '\n' || macAddress[i] == '\r')
+			break;
+
 		byte[0] = macAddress[i];
 		byte[1] = macAddress[i+1];
 
