@@ -1,7 +1,7 @@
  /*
   *  IVS Tools - Convert or merge IVs
   *
-  *  Copyright (C) 2006, 2007, 2008, 2009 Thomas d'Otreppe
+  *  Copyright (C) 2006-2018 Thomas d'Otreppe <tdotreppe@aircrack-ng.org>
   *  Copyright (C) 2004, 2005  Christophe Devine (pcap2ivs and mergeivs)
   *
   *  This program is free software; you can redistribute it and/or modify
@@ -43,6 +43,7 @@
 #include "uniqueiv.h"
 #include "osdep/byteorder.h"
 #include "common.h"
+#include "eapol.h"
 
 #define FAILURE -1
 #define IVS     1
@@ -68,18 +69,6 @@ struct AP_info
     int essid_stored;         /* essid stored in ivs file? */
 };
 
-struct WPA_hdsk
-{
-    uchar stmac[6];				 /* supplicant MAC               */
-    uchar snonce[32];			 /* supplicant nonce             */
-    uchar anonce[32];			 /* authenticator nonce          */
-    uchar keymic[16];			 /* eapol frame MIC              */
-    uchar eapol[256];			 /* eapol frame contents         */
-    int eapol_size;				 /* eapol frame size             */
-    int keyver;					 /* key version (TKIP / AES)     */
-    int state;					 /* handshake completion         */
-};
-
 /* linked list of detected clients */
 
 struct ST_info
@@ -103,20 +92,19 @@ struct globals
 }
 G;
 
-static uchar ZERO[32] =
+static unsigned char ZERO[32] =
         "\x00\x00\x00\x00\x00\x00\x00\x00"
         "\x00\x00\x00\x00\x00\x00\x00\x00"
         "\x00\x00\x00\x00\x00\x00\x00\x00"
         "\x00\x00\x00\x00\x00\x00\x00\x00";
 
-extern char * getVersion(char * progname, int maj, int min, int submin, int svnrev, int beta, int rc);
-
 void usage(int what)
 {
-    printf("\n  %s - (C) 2006, 2007, 2008, 2009 Thomas d\'Otreppe\n"
-            "  Original work: Christophe Devine\n"
-            "  http://www.aircrack-ng.org\n"
-            "\n   usage: ", getVersion("ivsTools", _MAJ, _MIN, _SUB_MIN, _REVISION, _BETA, _RC));
+    char *version_info = getVersion("ivsTools", _MAJ, _MIN, _SUB_MIN, _REVISION, _BETA, _RC);
+    printf("\n  %s - (C) 2006-2018 Thomas d\'Otreppe\n"
+            "  https://www.aircrack-ng.org\n"
+            "\n   usage: ", version_info);
+    free(version_info);
     if (what == 0 || what == 1)
         printf( "ivstools --convert <pcap file> <ivs output file>\n"
                 "        Extract ivs from a pcap file\n");
@@ -134,7 +122,6 @@ int merge( int argc, char *argv[] )
     unsigned char buffer[1024];
     FILE *f_in, *f_out;
     struct ivs2_filehdr fivs2;
-    struct ivs2_pkthdr ivs2;
 
     if( argc < 5 )
     {
@@ -142,6 +129,14 @@ int merge( int argc, char *argv[] )
         return( 1 );
     }
 
+    // Check filenames are not empty
+    for (i = 2; i < argc - 1; ++i) {
+        if (argv[i][0] == 0) {
+            printf("Filename #%d is empty, aborting\n", i - 1);
+            return ( 1 );
+        }
+    }
+
     printf( "Creating %s\n", argv[argc - 1] );
 
     if( ( f_out = fopen( argv[argc - 1], "wb+" ) ) == NULL )
@@ -158,36 +153,47 @@ int merge( int argc, char *argv[] )
 
         if( ( f_in = fopen( argv[i], "rb" ) ) == NULL )
         {
+        	fclose(f_out);
             perror( "fopen failed" );
             return( 1 );
         }
 
         if( fread( buffer, 1, 4, f_in ) != 4 )
         {
+        	fclose(f_out);
+        	fclose(f_in);
             perror( "fread file header failed" );
             return( 1 );
         }
 
         if( memcmp( buffer, IVSONLY_MAGIC, 4 ) == 0 )
         {
+        	fclose(f_out);
+        	fclose(f_in);
             printf( "%s is an old .ivs file\n", argv[i] );
             return( 1 );
         }
 
         if( memcmp( buffer, IVS2_MAGIC, 4 ) != 0 )
         {
+        	fclose(f_out);
+        	fclose(f_in);
             printf( "%s is not an .%s file\n", argv[i], IVS2_EXTENSION );
             return( 1 );
         }
 
         if( fread( &fivs2, 1, sizeof(struct ivs2_filehdr), f_in ) != (size_t) sizeof(struct ivs2_filehdr) )
         {
+        	fclose(f_out);
+        	fclose(f_in);
             perror( "fread file header failed" );
             return( 1 );
         }
 
         if( fivs2.version > IVS2_VERSION )
         {
+        	fclose(f_out);
+        	fclose(f_in);
             printf( "Error, wrong %s version: %d. Supported up to version %d.\n", IVS2_EXTENSION, fivs2.version, IVS2_VERSION );
             return( 1 );
         }
@@ -195,14 +201,14 @@ int merge( int argc, char *argv[] )
         if( i == 2 )
         {
             unused = fwrite( buffer, 1, 4, f_out );
-            unused = fwrite( &ivs2, 1, sizeof(struct ivs2_filehdr), f_out );
+            unused = fwrite( &fivs2, 1, sizeof(struct ivs2_filehdr), f_out );
         }
 
         while( ( n = fread( buffer, 1, 1024, f_in ) ) > 0 )
         {
             nbw += n;
             unused = fwrite( buffer, 1, n, f_out );
-            printf( "%ld bytes written\r", nbw );
+            printf( "%lu bytes written\r", nbw );
         }
 
         fclose( f_in );
@@ -215,9 +221,10 @@ int merge( int argc, char *argv[] )
     return( 0 );
 }
 
-int dump_add_packet( unsigned char *h80211, int caplen)
+int dump_add_packet( unsigned char *h80211, unsigned caplen)
 {
-    int i, n, z, seq, dlen, clen;
+    int i, n, seq, dlen, clen;
+    unsigned z;
     struct ivs2_pkthdr ivs2;
     unsigned char *p;
     unsigned char bssid[6];
@@ -698,13 +705,21 @@ skip_station:
                       ( h80211[z + 6] & 0x80 ) != 0 &&
                       ( h80211[z + 5] & 0x01 ) != 0 )
                 {
+                    st_cur->wpa.eapol_size = ( h80211[z + 2] << 8 )
+                            +   h80211[z + 3] + 4;
+
+                    if (st_cur->wpa.eapol_size > sizeof(st_cur->wpa.eapol_size) ||
+                        caplen - z < st_cur->wpa.eapol_size) {
+                        // ignore packet trying to crash us
+                        st_cur->wpa.eapol_size = 0;
+                        return 0;
+                    }
+
                     if( memcmp( &h80211[z + 17], ZERO, 32 ) != 0 )
                     {
                         memcpy( st_cur->wpa.anonce, &h80211[z + 17], 32 );
                         st_cur->wpa.state |= 4;
                     }
-                    st_cur->wpa.eapol_size = ( h80211[z + 2] << 8 )
-                            +   h80211[z + 3] + 4;
 
                     memcpy( st_cur->wpa.keymic, &h80211[z + 81], 16 );
                     memcpy( st_cur->wpa.eapol,  &h80211[z], st_cur->wpa.eapol_size );
@@ -793,6 +808,17 @@ int main( int argc, char *argv[] )
         return( 1 );
     }
 
+    // Check filenames are not empty
+    if (argv[2][0] == 0) {
+        printf("Invalid pcap file\n");
+        return ( 1 );
+    }
+
+    if (argv[3][0] == 0) {
+        printf("Invalid output file\n");
+        return ( 1 );
+    }
+
     memset( bssid_cur, 0, 6 );
     memset( bssid_prv, 0, 6 );
 
@@ -811,6 +837,7 @@ int main( int argc, char *argv[] )
     if( fread( &pfh, 1, n, f_in ) != (size_t) n )
     {
         perror( "fread(pcap file header) failed" );
+        fclose( f_in );
         return( 1 );
     }
 
@@ -819,6 +846,7 @@ int main( int argc, char *argv[] )
     {
         printf( "\"%s\" isn't a pcap file (expected "
                 "TCPDUMP_MAGIC).\n", argv[2] );
+        fclose( f_in );
         return( 1 );
     }
 
@@ -832,6 +860,7 @@ int main( int argc, char *argv[] )
     {
         printf( "\"%s\" isn't a regular 802.11 "
                 "(wireless) capture.\n", argv[2] );
+        fclose( f_in );
         return( 1 );
     }
 
@@ -842,6 +871,7 @@ int main( int argc, char *argv[] )
     if( ( G.f_ivs = fopen( argv[3], "wb+" ) ) == NULL )
     {
         perror( "fopen failed" );
+        fclose( f_in );
         return( 1 );
     }
 
@@ -859,7 +889,7 @@ int main( int argc, char *argv[] )
     {
         if( time( NULL ) - tt > 0 )
         {
-            printf( "\33[KRead %ld packets...\r", nbr );
+            printf( "\33[KRead %lu packets...\r", nbr );
             fflush( stdout );
             tt = time( NULL );
         }
@@ -871,8 +901,10 @@ int main( int argc, char *argv[] )
         if( fread( &pkh, 1, n, f_in ) != (size_t) n )
             break;
 
-        if( pfh.magic == TCPDUMP_CIGAM )
+        if( pfh.magic == TCPDUMP_CIGAM ) {
             SWAP32( pkh.caplen );
+            SWAP32( pkh.len );
+        }
 
         n = pkh.caplen;
 
@@ -945,10 +977,10 @@ int main( int argc, char *argv[] )
     fclose( f_in );
     fclose( G.f_ivs );
 
-    printf( "\33[2KRead %ld packets.\n", nbr );
+    printf( "\33[2KRead %lu packets.\n", nbr );
 
     if ( nbivs > 0 )
-        printf( "Written %ld IVs.\n", nbivs);
+        printf( "Written %lu IVs.\n", nbivs);
     else
     {
         remove ( argv[3] );
