@@ -1,7 +1,7 @@
  /*
   * Server for osdep network driver.  Uses osdep itself!  [ph33r teh recursion]
   *
-  *  Copyright (C) 2006, 2007, 2008, 2009 Thomas d'Otreppe
+  *  Copyright (C) 2006-2018 Thomas d'Otreppe <tdotreppe@aircrack-ng.org>
   *  Copyright (C) 2004, 2005 Christophe Devine
   *
   *  This program is free software; you can redistribute it and/or modify
@@ -32,15 +32,13 @@
 
 #define NULL_MAC "\x00\x00\x00\x00\x00\x00"
 
-extern char * getVersion(char * progname, int maj, int min, int submin, int svnrev, int beta, int rc);
 extern unsigned char * getmac(char * macAddress, int strict, unsigned char * mac);
 
 char usage[] =
 
 "\n"
-"  %s - (C) 2006, 2007, 2008, 2009 Thomas d\'Otreppe\n"
-"  Original work: Christophe Devine\n"
-"  http://www.aircrack-ng.org\n"
+"  %s - (C) 2006-2018 Thomas d\'Otreppe\n"
+"  https://www.aircrack-ng.org\n"
 "\n"
 "  usage: makeivs-ng [options]\n"
 "\n"
@@ -183,8 +181,12 @@ int main( int argc, char *argv[] )
 
 				paramUsed = 1;
                 sscanf(optarg, "%f", &errorrate);
+#if defined(__x86_64__) && defined(__CYGWIN__)
+                if (errorrate < 0.0f || errorrate > (0.0f + 100)) {
+#else
                 if (errorrate < 0.0f || errorrate > 100.0f) {
-					printf( usage, getVersion("makeivs-ng", _MAJ, _MIN, _SUB_MIN, _REVISION, _BETA, _RC)  );
+#endif
+			printf( usage, getVersion("makeivs-ng", _MAJ, _MIN, _SUB_MIN, _REVISION, _BETA, _RC)  );
                     printf( "Specified errorrate is invalid. [0-100]" );
                     return( 1 );
                 }
@@ -194,8 +196,12 @@ int main( int argc, char *argv[] )
             case 'd':
 
 				paramUsed = 1;
-                sscanf(optarg, "%f", &dupe);
-                if (dupe < 0.0f || dupe > 100.0f) {
+                if (sscanf(optarg, "%f", &dupe) != 1 || dupe < 0.0f || dupe >
+#if defined(__x86_64__) && defined(__CYGWIN__)
+			(0.0f + 100)) {
+#else
+			100.0f) {
+#endif
 					printf( usage, getVersion("makeivs-ng", _MAJ, _MIN, _SUB_MIN, _REVISION, _BETA, _RC)  );
 
                     printf( "Specified dupe is invalid. [0-100]" );
@@ -415,7 +421,12 @@ usage:
     pre_n=0;
     for( n = 0; n < count; n++ )
     {
-        if( (dupe==0) || (pre_n == n) || ((float)rand()/(float)RAND_MAX > (float)((float)dupe/100.0f)) )
+        if( (dupe==0) || (pre_n == n) || ((float)rand()/(float)RAND_MAX > (float)((float)dupe/
+#if defined(__x86_64__) && defined(__CYGWIN__)
+		(0.0f + 100))) )
+#else
+		100.0f)) )
+#endif
         {
             if(prng)
             {
@@ -466,7 +477,12 @@ usage:
             SWAP( S[i], S[j] );
         }
 
-        if(errorrate > 0 && ((float)((float)rand()/(float)RAND_MAX) <= (float)(errorrate/100.0f)) )
+        if(errorrate > 0 && ((float)((float)rand()/(float)RAND_MAX) <= (float)(errorrate/
+#if defined(__x86_64__) && defined(__CYGWIN__)
+		(0.0f + 100))) )
+#else
+		100.0f)) )
+#endif
         {
             SWAP( S[1], S[11] );
         }
@@ -504,7 +520,12 @@ usage:
             fprintf( f_ivs_out, "%c", S[(S[i] + S[j]) & 0xFF] );
         }
         if((n%10000) == 0)
-            printf("%2.1f%%\r", ((float)n/(float)count)*100.0f);
+            printf("%2.1f%%\r", ((float)n/(float)count)*
+#if defined(__x86_64__) && defined(__CYGWIN__)
+		(0.0f + 100));
+#else
+		100.0f);
+#endif
         fflush(stdout);
     }
 
