@@ -1,8 +1,34 @@
-/*-
+/* 
  * Copyright (c) 2007, 2008, Andrea Bittau <a.bittau@cs.ucl.ac.uk>
- *
  * All OS dependent crap should go here.
  *
+ * This program is free software; you can redistribute it and/or
+ * modify it under the terms of the GNU General Public License as
+ * published by the Free Software Foundation; either version 2 of the
+ * License, or (at your option) any later version.
+ *
+ * This program is distributed in the hope that it will be useful, but
+ * is provided AS IS, WITHOUT ANY WARRANTY; without even the implied
+ * warranty of MERCHANTABILITY, FITNESS FOR A PARTICULAR PURPOSE, and
+ * NON-INFRINGEMENT.  See the GNU General Public License for more details.
+ *
+ * You should have received a copy of the GNU General Public License
+ * along with this program; if not, write to the Free Software
+ * Foundation, Inc., 59 Temple Place - Suite 330, Boston,
+ * MA 02111-1307, USA.
+ *
+ * In addition, as a special exception, the copyright holders give
+ * permission to link the code of portions of this program with the
+ * OpenSSL library under certain conditions as described in each
+ * individual source file, and distribute linked combinations
+ * including the two.
+ * You must obey the GNU General Public License in all respects
+ * for all of the code used other than OpenSSL.  If you modify
+ * file(s) with this exception, you may extend this exception to your
+ * version of the file(s), but you are not obligated to do so.  If you
+ * do not wish to do so, delete this exception statement from your
+ * version.  If you delete this exception statement from all source
+ * files in the program, then also delete it here.
  */
 
 #ifndef __AIRCRACK_NG_OSEDEP_H__
@@ -14,6 +40,22 @@
 #include "byteorder.h"
 #include "packed.h"
 
+#if defined(__APPLE_CC__) && defined(_XCODE)
+#include <pcap/bpf.h>
+#undef	LINKTYPE_RADIOTAP_HDR
+#define LINKTYPE_RADIOTAP_HDR   DLT_IEEE802_11_RADIO
+#undef	LINKTYPE_IEEE802_11
+#define LINKTYPE_IEEE802_11     DLT_IEEE802_11
+#undef	LINKTYPE_PRISM_HEADER
+#define LINKTYPE_PRISM_HEADER   DLT_PRISM_HEADER
+#undef	LINKTYPE_ETHERNET
+#define LINKTYPE_ETHERNET       DLT_ERF_ETH
+#undef	LINKTYPE_PPI_HDR
+#define LINKTYPE_PPI_HDR        DLT_PPI
+#undef	TCPDUMP_MAGIC
+#define TCPDUMP_MAGIC           0xa1b2c3d4
+#endif
+
 /* For all structures, when adding new fields, always append them to the end.
  * This way legacy binary code does not need to be recompiled.  This is
  * particularly useful for DLLs.  -sorbo
@@ -43,6 +85,7 @@ struct wif {
                            struct rx_info *ri);
         int     (*wi_write)(struct wif *wi, unsigned char *h80211, int len,
                             struct tx_info *ti);
+        int     (*wi_set_ht_channel)(struct wif *wi, int chan, unsigned int htval);
         int     (*wi_set_channel)(struct wif *wi, int chan);
         int     (*wi_get_channel)(struct wif *wi);
         int     (*wi_set_freq)(struct wif *wi, int freq);
@@ -68,6 +111,7 @@ extern int wi_read(struct wif *wi, unsigned char *h80211, int len,
 extern int wi_write(struct wif *wi, unsigned char *h80211, int len,
 		    struct tx_info *ti);
 extern int wi_set_channel(struct wif *wi, int chan);
+extern int wi_set_ht_channel(struct wif *wi, int chan, unsigned int htval);
 extern int wi_get_channel(struct wif *wi);
 extern int wi_set_freq(struct wif *wi, int freq);
 extern int wi_get_freq(struct wif *wi);
@@ -100,7 +144,7 @@ extern int get_battery_state(void);
 
 /* Client code can create a tap interface */
 /* XXX we can unify the tap & wi stuff in the future, but for now, lets keep
- * them seperate until we learn something.
+ * them separate until we learn something.
  */
 struct tif {
 	int	(*ti_read)(struct tif *ti, void *buf, int len);
