@@ -1,5 +1,5 @@
 /*
- *  Copyright (C) 2005, 2006, 2007, 2008, 2009 Andrea Bittau <a.bittau@cs.ucl.ac.uk>
+ *  Copyright (C) 2005-2009 Andrea Bittau <a.bittau@cs.ucl.ac.uk>
  *
  *  This program is free software; you can redistribute it and/or modify
  *  it under the terms of the GNU General Public License as published by
@@ -33,7 +33,7 @@
 
 #include <sys/types.h>
 #include <sys/socket.h>
-#include <sys/termios.h>
+#include <termios.h>
 #include <sys/ioctl.h>
 #include <sys/stat.h>
 #include <sys/wait.h>
@@ -90,8 +90,6 @@
 #define PRGA_FILE "prga.log"
 #define KEYLIMIT 1000000
 
-extern char * getVersion(char * progname, int maj, int min, int submin, int svnrev, int beta, int rc);
-
 struct frag_state {
 	struct ieee80211_frame	fs_wh;
 	unsigned char		*fs_data;
@@ -199,7 +197,7 @@ void show_wep_stats( int B, int force, PTW_tableentry table[PTW_KEYHSBYTES][PTW_
     static int is_cleared = 0;
 
     return;
-
+    /*
     if( ioctl( 0, TIOCGWINSZ, &ws ) < 0 )
     {
         ws.ws_row = 25;
@@ -255,6 +253,7 @@ void show_wep_stats( int B, int force, PTW_tableentry table[PTW_KEYHSBYTES][PTW_
 //        printf( "\33[J" );
 
     printf( "\n" );
+    */
 }
 
 static struct wstate *get_ws(void)
@@ -1072,6 +1071,7 @@ static void got_ip(struct wstate *ws)
 	strncpy(ws->ws_netip, inet_ntoa(*in), 16-1);
 
 	time_print("Got IP=(%s)\n", ws->ws_netip);
+	memset(ws->ws_myip, 0, sizeof(ws->ws_myip));
 	strncpy(ws->ws_myip, ws->ws_netip, sizeof(ws->ws_myip)-1);
 
 	ptr = strchr(ws->ws_myip, '.');
@@ -1448,7 +1448,7 @@ static void decrypt(struct wstate *ws)
 
 	// guess diff prga byte...
 	if (ws->ws_dfs.fs_waiting_relay) {
-		unsigned short* seq;
+		unsigned short seq;
 		ws->ws_dpi.pi_prga[ws->ws_dpi.pi_len-1]++;
 
 		ws->ws_dfs.fs_wh.i_addr3[5] =
@@ -1457,8 +1457,11 @@ static void decrypt(struct wstate *ws)
 		ws->ws_dfs.fs_waiting_relay = 0;
 		ws->ws_dfs.fs_ptr = ws->ws_dfs.fs_data;
 
-		seq = (unsigned short*) &ws->ws_dfs.fs_wh.i_seq;
-		*seq = fnseq(0, ws->ws_psent);
+		seq = fnseq(0, ws->ws_psent);
+		ws->ws_dfs.fs_wh.i_seq[0] = (u_int8_t)(seq >> 8);
+		ws->ws_dfs.fs_wh.i_seq[1] = (u_int8_t)(seq % 256);
+		//seq = (unsigned short*) &ws->ws_dfs.fs_wh.i_seq;
+		//*seq = fnseq(0, ws->ws_psent);
 	}
 
 	send_fragment(ws, &ws->ws_dfs,
@@ -1610,7 +1613,7 @@ static int do_crack(struct wstate *ws)
         int (* all)[256];
         int i,j;
 
-        all = malloc(256*32*sizeof(int));
+        all = malloc(32*sizeof(int [256]));
         if (all == NULL) {
             return 1;
         }
@@ -2032,10 +2035,10 @@ static void start(struct wstate *ws, char *dev)
 static void usage(char* pname)
 {
 	if (pname) {}
-
+    char *version_info = getVersion("Wesside-ng", _MAJ, _MIN, _SUB_MIN, _REVISION, _BETA, _RC);
 	printf("\n"
 		"  %s - (C) 2007, 2008, 2009 Andrea Bittau\n"
-		"  http://www.aircrack-ng.org\n"
+		"  https://www.aircrack-ng.org\n"
 		"\n"
 		"  Usage: wesside-ng <options>\n"
 		"\n"
@@ -2053,8 +2056,8 @@ static void usage(char* pname)
 		"       -f   <max chan> : Highest scanned chan (default: 11)\n"
 		"       -k      <txnum> : Ignore acks and tx txnum times\n"
 		"\n",
-		getVersion("Wesside-ng", _MAJ, _MIN, _SUB_MIN, _REVISION, _BETA, _RC));
-
+		version_info);
+    free(version_info);
 	exit(0);
 }
 
